## Precept 7 Empty Code 
## Many thanks to Clark Bernier and Elisha Cohen

setwd("~/Dropbox/Soc500/2016 Materials/Precepts/Precept 7")

##--------------------------------------------
## A Warm-up: Basic Matrix Operations                  
##--------------------------------------------
## Let's do these on the board and in R! 

A <- matrix(c(1,2,3,4), nrow = 2, ncol = 2, byrow = TRUE)
A
B <- matrix(c(0,1,0,1), nrow = 2, ncol = 2, byrow = TRUE)
B
C <- matrix(c(6,5,4,3,2,1), nrow = 3, ncol = 2, byrow = TRUE)
C

A + B
A - B # matrix subtraction

A %*% B # matrix multiplication
B %*% A # note not commutative! 

C %*% A 
A %*% C # not conformable! :( 

solve(A) # matrix inverse
t(C) # transpose

# Diag three ways:
diag(3) # when given a constant k, gives a k by k identity matrix
diag(A) # when given a matrix A, gives a vector with diagonal of matrix A

##--------------------------------------------
## F-Tests (restricted and unrestricted models)
##--------------------------------------------

# Let's revisit our trusty old subprime data set! (Sorry!)
subprime <- read.csv("subprime.csv")

# Let's say we have the following full model:
full_model <- lm(loan.amount ~ income + black + woman + black:woman, data = subprime)

summary(full_model)

# What's an F test used for?

# What's the null hypthesis?
#     Adding the new variables doesn't explain more of the dependent 
#     variable than the rest of the model
#     or that certain variables are not different from each other

# To do an f-test, we need to compare 
#     the restricted model 
#     to the unrestricted model (contains the all variable(s) of interest)

# Let's do a couple F tests 

# 1) An omnibus test = do any of the X variables help predict Y?
restricted_model1 <- # what should we put here?
summary(restricted_model1)
mean(subprime$loan.amount)
  
# 2) A test to see if we should include woman and the interaction term
restricted_model2 <- # what should we put here?
summary(restricted_model2)  

# Grab the SSR of both models
SSR_full <- sum(resid(full_model) ^ 2)
# what about for the others? 

# And the degrees of freedom of the unrestriced model
df_full <- df.residual(full_model)

# What is the df diff?
df_diff1 <- 4
df_diff2 <- 2

# FIRST TEST
F <- ((SSR_res - SSR_full) / df_diff1) / (SSR_full / df_full)
F
?pf

pf(F, df_diff1, df_full, lower.tail = FALSE) # for p-value

# We can check this result against the lm() output
summary(full_model)$fstatistic

# SECOND RESULT
F <- ((SSR_res2 - SSR_full) / df_diff2) / (SSR_full / df_full)
F

qf(0.99, df_diff2, df_full) # critical value for alpha = 0.01

pf(F, df_diff2, df_full, lower.tail = FALSE)

# An alternative is to do this with ANOVA 
anova(restricted_model1, full_model)
anova(restricted_model2, full_model)

##--------------------------------------------
## Bootstrap to get SE and confidence intervals
##--------------------------------------------

library(dplyr)
library(ggplot2)

## Let's say we have a sample of 5000 observations from the full subprime population dataset 
## We want to use bootstrapping to get standard errors for the sample mean of loan.amount
## Let's first implement this with a for loop

set.seed(12334)
# First create the random sample
n <- 5000
random_sample <- sample_n(subprime, n)
mean(random_sample$loan.amount)

# We can do it by for loop
reps <- 1000
boot_vec <- rep(0, reps)
for (i in 1:reps) {
  boot_samp <- sample_n(random_sample, nrow(random_sample), replace = T)
  boot_vec[i] <- mean(boot_samp$loan.amount)
}
boot_vec
# Or replicate
calc_boot <- function() {
  # what goes in here? 
}
boot_vec <- replicate(reps, calc_boot())

# the distribution of our results
ggplot() + geom_density(aes(x = boot_vec))

# their mean and standard deviation
mean(boot_vec) # about the same as the true mean in our sample
sd(boot_vec)

# and the 95% confidence interval
quantile(boot_vec, probs = c(0.025, 0.975))

# How does our bootstrap SE compare to the true SE? 
sd(boot_vec)
sd(subprime$loan.amount) / sqrt(n)   # not bad!! 



## We can use bootstrapping to find standard errors for lots of things
## Let's return to our previous full model
## And try to retrieve the standard errors of our coefficients 
reps <- 1000
set.seed(12334)
boot_coefs <- function(data) {
  # how should I do this? 
}
boot_coefs_out <- replicate(reps, boot_coefs(random_sample))
View(boot_coefs_out)

# Get mean and variance
apply(boot_coefs_out, 1, mean)
apply(boot_coefs_out, 1, sd)

# And the 95% confidence interval
apply(boot_coefs_out, 1, quantile, probs = c(0.025, 0.975))

# How does this compare to the estimates we get? 
full_model <- lm(loan.amount ~ income + black + woman + black:woman, data = random_sample)

summary(full_model)$coefficients
sqrt(diag(vcov(full_model))) # gets values from variance-covariance matrix 


